@@ -9,7 +9,7 @@ use POSIX qw(locale_h);
 setlocale(LC_NUMERIC,"C");
 
 
-$VERSION = '21.1';
+$VERSION = '22.1';
 
 # Some function might be excluded from export and assessment.
 our @EXCLUDED_FUNCTION = ('SQUIRREL_GET_ERROR_OFFSET');
@@ -306,7 +306,7 @@ FROM INFORMATION_SCHEMA.COLUMNS
 $condition
 ORDER BY ORDINAL_POSITION};
 	# Version below 5.5 do not have DATA_TYPE column it is named DTD_IDENTIFIER
-	if ($self->{db_version} lt '5.5.0') {
+	if ($self->{db_version} < '5.5.0') {
 		$str =~ s/\bDATA_TYPE\b/DTD_IDENTIFIER/;
 	}
 	my $sth = $self->{dbh}->prepare($str);
@@ -707,7 +707,7 @@ sub _get_functions
 	# COLLATION_CONNECTION     | varchar(32)   | NO   |     |                     |       |
 	# DATABASE_COLLATION       | varchar(32)   | NO   |     |                     |       |
 
-	my $str = "SELECT ROUTINE_NAME,ROUTINE_DEFINITION,DATA_TYPE,ROUTINE_BODY,EXTERNAL_LANGUAGE,SECURITY_TYPE,IS_DETERMINISTIC FROM INFORMATION_SCHEMA.ROUTINES";
+	my $str = "SELECT ROUTINE_NAME,ROUTINE_DEFINITION,DATA_TYPE,ROUTINE_BODY,EXTERNAL_LANGUAGE,SECURITY_TYPE,IS_DETERMINISTIC,ROUTINE_TYPE FROM INFORMATION_SCHEMA.ROUTINES";
 	if ($self->{schema}) {
 		$str .= " AND ROUTINE_SCHEMA = '$self->{schema}'";
 	}
@@ -715,28 +715,27 @@ sub _get_functions
 	$str =~ s/ AND / WHERE /;
 	$str .= " ORDER BY ROUTINE_NAME";
 	# Version below 5.5 do not have DATA_TYPE column it is named DTD_IDENTIFIER
-	if ($self->{db_version} lt '5.5.0') {
+	if ($self->{db_version} < '5.5.0') {
 		$str =~ s/\bDATA_TYPE\b/DTD_IDENTIFIER/;
 	}
 	my $sth = $self->{dbh}->prepare($str) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	$sth->execute(@{$self->{query_bind_params}}) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 
 	my %functions = ();
-	while (my $row = $sth->fetch) {
-
-		my $kind = 'FUNCTION';
-		if (!$row->[2]) {
-			$kind = 'PROCEDURE';
-		}
+	while (my $row = $sth->fetch)
+	{
+		my $kind = $row->[7]; # FUNCTION or PROCEDURE
 		next if ( ($kind ne $self->{type}) && ($self->{type} ne 'SHOW_REPORT') );
 		my $sth2 = $self->{dbh}->prepare("SHOW CREATE $kind $row->[0]") or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 		$sth2->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
-		while (my $r = $sth2->fetch) {
+		while (my $r = $sth2->fetch)
+		{
 			$functions{"$row->[0]"}{text} = $r->[2];
 			last;
 		}
 		$sth2->finish();
-		if ($self->{plsql_pgsql} || ($self->{type} eq 'SHOW_REPORT')) {
+		if ($self->{plsql_pgsql} || ($self->{type} eq 'SHOW_REPORT'))
+		{
 			$functions{"$row->[0]"}{name} = $row->[0];
 			$functions{"$row->[0]"}{return} = $row->[2];
 			$functions{"$row->[0]"}{definition} = $row->[1];
@@ -960,7 +959,7 @@ sub _list_all_funtions
 	if ($self->{schema}) {
 		$str .= " AND ROUTINE_SCHEMA = '$self->{schema}'";
 	}
-	if ($self->{db_version} lt '5.5.0') {
+	if ($self->{db_version} < '5.5.0') {
 		$str =~ s/\bDATA_TYPE\b/DTD_IDENTIFIER/;
 	}
 	$str .= " " . $self->limit_to_objects('FUNCTION','ROUTINE_NAME');
@@ -1413,10 +1412,7 @@ sub _get_objects
 		}
 	}
 	# FUNCTION
-	$sql = "SELECT ROUTINE_NAME FROM INFORMATION_SCHEMA.ROUTINES WHERE DATA_TYPE IS NOT NULL AND ROUTINE_SCHEMA = '$self->{schema}'";
-	if ($self->{db_version} lt '5.5.0') {
-		$sql =~ s/\bDATA_TYPE\b/DTD_IDENTIFIER/;
-	}
+	$sql = "SELECT ROUTINE_NAME FROM INFORMATION_SCHEMA.ROUTINES WHERE ROUTINE_TYPE = 'FUNCTION' AND ROUTINE_SCHEMA = '$self->{schema}'";
 	$sth = $self->{dbh}->prepare( $sql ) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	while ( my @row = $sth->fetchrow()) {
@@ -1424,10 +1420,7 @@ sub _get_objects
 	}
 	$sth->finish();
 	# PROCEDURE
-	$sql = "SELECT ROUTINE_NAME FROM INFORMATION_SCHEMA.ROUTINES WHERE DATA_TYPE IS NULL AND ROUTINE_SCHEMA = '$self->{schema}'";
-	if ($self->{db_version} lt '5.5.0') {
-		$sql =~ s/\bDATA_TYPE\b/DTD_IDENTIFIER/;
-	}
+	$sql = "SELECT ROUTINE_NAME FROM INFORMATION_SCHEMA.ROUTINES WHERE ROUTINE_TYPE = 'PROCEDURE' AND ROUTINE_SCHEMA = '$self->{schema}'";
 	$sth = $self->{dbh}->prepare( $sql ) or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	$sth->execute or $self->logit("FATAL: " . $self->{dbh}->errstr . "\n", 0, 1);
 	while ( my @row = $sth->fetchrow()) {
